import json
import os
import requests
import subprocess
import sys
import time

class ZbxLib:
    ROBOT_LIBRARY_SCOPE = 'TEST SUITE'
    ROBOT_EXIT_ON_FAILURE = True
    
    def __init__(self, url):
        self.__id = 1
        self.__url = url
        self.__auth = None
        return

    def __zbxcall(self, payload):
        headers = {'Content-Type': 'application/json-rpc'}
        request = requests.post(self.__url, headers=headers, data=payload)

        print("Request:")
        print(payload)
        print()
        print("Response:")
        print(request.text)

        self.__id += 1

        resp = json.loads(request.text)

        if 'error' in resp:
            raise Exception(resp['error']['data'])

        return resp

    #
    # Utilities.
    #
    def wait_for_configuration_sync(self, seconds):
        print("Waiting for configuration to sync: %s seconds" % (seconds))
        time.sleep(float(seconds))

    def __call_zabbix_sender(self, zabbix_sender, server_ip, host_name, item_key, item_value):
        cmd = [
            zabbix_sender, 
           "-z", server_ip,
            "-s", host_name,
            "-k", item_key,
            "-o", item_value
        ]
       
        print()
        print("Command: %s" % (cmd))
                
        retcode = subprocess.call(cmd, shell=False)

        print("Retcode: %d" % (retcode))
        print()

        return retcode

    def call_zabbix_sender_and_expect_success(self, zabbix_sender, server_ip, host_name, item_key, item_value):
        retcode = self.__call_zabbix_sender(zabbix_sender, server_ip, host_name, item_key, item_value)

        if retcode != 0:
            raise Exception("Command failed!")

        # Let the server process the value before continuing.
        time.sleep(1)

    def call_zabbix_sender_and_expect_failure(self, zabbix_sender, server_ip, host_name, item_key, item_value):
        retcode = self.__call_zabbix_sender(zabbix_sender, server_ip, host_name, item_key, item_value)

        if retcode == 0:
            raise Exception("Command succeeded!")
        
    #
    # API services.
    #
    def api_version_is(self, expected_api_version):
        req = """
        {
            "jsonrpc": "2.0",
            "method": "apiinfo.version",
            "params": {},
            "id": %d,
            "auth": null 
        }""" % (self.__id)

        resp = self.__zbxcall(req)

        actual_api_version = resp['result']

        print()
        print("Expected API version: %s" % (expected_api_version))
        print("Actual API version: %s" % (actual_api_version))
        print()

        if actual_api_version != expected_api_version:
            raise Exception("expected: %s, got %s" % (expected_api_version, actual_api_version))

    #
    # User sessions.
    #
    def login(self, user, password):
        req = """
        {
            "jsonrpc": "2.0",
            "method": "user.login",
            "id": %d,
            "params": {
                "user": "%s",
                "password": %s
            },
            "auth": null 
        }""" % (self.__id, user, json.dumps(password))

        # Already logged in? A single session per library is supported...
        if self.__auth:
            raise Exception("Already logged in to %s." % (self.__url))

        resp = self.__zbxcall(req)

        self.__auth = resp['result']

    def logout(self):
        req = """
        {
            "jsonrpc": "2.0",
            "method": "user.logout",
            "params": {},
            "id": %d,
            "auth": "%s" 
        }""" % (self.__id, self.__auth)

        resp = self.__zbxcall(req)

    #
    # Host groups.
    #
    def create_host_group(self, host_group_name):
        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "hostgroup.create",
            "params": {
                "name": %s
            }
        }""" % (self.__id, self.__auth, json.dumps(host_group_name))

        resp = self.__zbxcall(req)

    def __get_host_group_id(self, host_group_name):
        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "hostgroup.get",
            "params": {
                "output": "groupid",
                "filter": {
                    "name": [
                        %s
                    ]
                }
            }
        }""" % (self.__id, self.__auth, json.dumps(host_group_name))

        resp = self.__zbxcall(req)

        print()
        print("Host group name: %s" % (host_group_name))

        # Host group exists?
        if len(resp['result']) == 0:
            raise Exception("Host group does not exist!")
               
        host_group_id = resp['result'][0]['groupid']
        
        print("Host group id: %s" % (host_group_id))
        print()

        return host_group_id

    def delete_host_group(self, host_group_name):
        host_group_id = self.__get_host_group_id(host_group_name)
        
        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "hostgroup.delete",
            "params": [
                "%s"
            ]
        }""" % (self.__id, self.__auth, host_group_id)

        resp = self.__zbxcall(req)

    #
    # Hosts.
    #
    def create_host(self, host_name, host_group_name):
        host_group_id = self.__get_host_group_id(host_group_name)

        # Unfortunately host cannot be created without an interface (that
        # could be added later).
        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "host.create",
            "params": {
                "host": %s,
                "interfaces": [
                    {
                        "type": 1,
                        "main": 1,
                        "useip": 1,
                        "ip": "0.0.0.0",
                        "dns": "",
                        "port": "0"
                    }
                ],
                "groups": [
                    {
                        "groupid": "%s"
                    }
                ]
            }
        }""" % (self.__id, self.__auth, json.dumps(host_name), host_group_id)

        resp = self.__zbxcall(req)

    def __get_host_id(self, host_name):
        req = """{
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "host.get",
            "params": {
                "output": "hostid",
                "filter": {
                    "host": [
                        %s
                    ]
                }
            }
        }""" % (self.__id, self.__auth, json.dumps(host_name))

        resp = self.__zbxcall(req)

        print()
        print("Host name: %s" % (host_name))

        # Host exists?
        if len(resp['result']) == 0:
            raise Exception("Host does not exist!")

        host_id = resp['result'][0]['hostid']
        
        print("Host id: %s" % (host_id))
        print()

        return host_id

    def delete_host(self, host_name):
        host_id = self.__get_host_id(host_name)

        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "host.delete",
            "params": [
                "%s"
            ]
        }""" % (self.__id, self.__auth, host_id)

        resp = self.__zbxcall(req)

    #
    # Items.
    #
    def __get_item_type_num(self, item_type):
        mapping = {
            'Zabbix agent':          0,
            'SNMPv1 agent':          1,
            'Zabbix trapper':        2,
            'simple check':          3,
            'SNMPv2 agent':          4,
            'Zabbix internal':       5,
            'SNMPv3 agent':          6,
            'Zabbix agent (active)': 7,
            'Zabbix aggregate':      8,
            'web item':              9,
            'external check':        10,
            'database monitor':      11,
            'IPMI agent':            12,
            'SSH agent':             13,
            'TELNET agent':          14,
            'calculated':            15,
            'JMX agent':             16,
            'SNMP trap':             17
        }

        print()
        print("Item type: %s" % (item_type))
        print("Item type num: %s" % (mapping[item_type]))
        print()

        return mapping[item_type]

    def __get_data_type_num(self, data_type):
        mapping = {
            'numeric float':         0,
            'character':             1,
            'log':                   2,
            'numeric unsigned':      3,
            'text':                  4
        }
        
        print()
        print("Data type: %s" % (data_type))
        print("Data type num: %s" % (mapping[data_type]))
        print()

        return mapping[data_type]

    def create_item(self, host_name, item_name, item_key, item_type, data_type):
        host_id       = self.__get_host_id(host_name)
        item_type_num = self.__get_item_type_num(item_type)
        data_type_num = self.__get_data_type_num(data_type)

        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "item.create",
            "params": {
                "hostid": "%s",
                "name": %s,
                "key_": %s,
                "type": %d,
                "value_type": %d
            }
        }""" % (self.__id, self.__auth, host_id, json.dumps(item_name), json.dumps(item_key),
                item_type_num, data_type_num)

        resp = self.__zbxcall(req)

    def __get_item_id(self, host_name, item_key):
        host_id = self.__get_host_id(host_name)
        
        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "item.get",
            "params": {
                "output": "itemid",
                "hostids": "%s",
                "search": {
                    "key_": %s
                }            }
        }""" % (self.__id, self.__auth, host_id, json.dumps(item_key))

        resp = self.__zbxcall(req)

        print()
        print("Host name: %s" % (host_name))
        print("Item key: %s" % (item_key))

        # Item exists?
        if len(resp['result']) == 0:
            raise Exception("Item does not exist!")

        item_id = resp['result'][0]['itemid']
        
        print("Item id: %s" % (item_id))
        print()

        return item_id

    def delete_item(self, host_name, item_key):
        item_id = self.__get_item_id(host_name, item_key)
        
        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "item.delete",
            "params": [
                "%s"
            ]
        }""" % (self.__id, self.__auth, item_id)

        resp = self.__zbxcall(req)

    #
    # Triggers.
    #
    def create_trigger(self, trigger_name, trigger_expression):
        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "trigger.create",
            "params": {
                "description": %s,
                "expression": %s
            }
        }""" % (self.__id, self.__auth, json.dumps(trigger_name), json.dumps(trigger_expression))
        
        print()
        print("Trigger name: %s" % (trigger_name))
        print("Trigger expression: %s" % (trigger_expression))
        print()

        resp = self.__zbxcall(req)

    def __get_trigger_id(self, trigger_name):      
        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "trigger.get",
            "params": {
                "output": "triggerid",
                "search": {
                    "description": %s
                }
            }
        }""" % (self.__id, self.__auth, json.dumps(trigger_name))

        resp = self.__zbxcall(req)

        print()
        print("Trigger name: %s" % (trigger_name))
        
        # Trigger exists?
        if len(resp['result']) == 0:
            raise Exception("Trigger does not exist!")

        # Triggers are independent of hosts, items and thus can be "duplicated".
        if len(resp['result']) > 1:
            raise Exception("Multiple triggers with same name!!")

        trigger_id = resp['result'][0]['triggerid']
        
        print("Trigger id: %s" % (trigger_id))
        print()

        return trigger_id

    def set_trigger_expression(self, trigger_name, trigger_expression):
        trigger_id = self.__get_trigger_id(trigger_name);
        
        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "trigger.update",
            "params": {
                "triggerid": "%s",
                "expression": %s
            }
        }""" % (self.__id, self.__auth, trigger_id, json.dumps(trigger_expression))

        resp = self.__zbxcall(req)

    def set_trigger_correlation_mode_and_tag(self, trigger_name, tag_name):
        trigger_id = self.__get_trigger_id(trigger_name);
        
        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "trigger.update",
            "params": {
                "triggerid": "%s",
                "type": "1",
                "correlation_mode": "1",
                "correlation_tag": %s
            }
        }""" % (self.__id, self.__auth, trigger_id, json.dumps(tag_name))

        resp = self.__zbxcall(req)

    def __get_trigger_tags(self, trigger_name):
        trigger_id = self.__get_trigger_id(trigger_name)

        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "trigger.get",
            "params": {
                "output": "triggerid",
                "selectTags": "extend",
                "triggerids": [ "%s" ]
            }
        }""" % (self.__id, self.__auth, trigger_id)

        resp = self.__zbxcall(req)

        return resp['result'][0]['tags']

    def __add_tag_with_value_to_trigger(self, trigger_name, tag_name, tag_value):
        trigger_id = self.__get_trigger_id(trigger_name)

        tags = self.__get_trigger_tags(trigger_name)

        # Add the new tag to the list of existing tags.
        # Otherwise the list of tags for the trigger is reset and contains
        # only the new tag.
        tv = {'tag': tag_name, 'value': tag_value}
        tags.append(tv)
        
        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "trigger.update",
            "params": {
                "triggerid": "%s",
                "tags": %s
            }
        }""" % (self.__id, self.__auth, trigger_id, json.dumps(tags))

        resp = self.__zbxcall(req)

    def add_tag_to_trigger(self, trigger_name, tag_name):
        self.__add_tag_with_value_to_trigger(trigger_name, tag_name, "")

    def add_tag_with_value_to_trigger(self, trigger_name, tag_name, tag_value):
        self.__add_tag_with_value_to_trigger(trigger_name, tag_name, tag_value)

    def delete_trigger(self, trigger_name):
        trigger_id = self.__get_trigger_id(trigger_name)
        
        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "trigger.delete",
            "params": [
                "%s"
            ]
        }""" % (self.__id, self.__auth, trigger_id)

        resp = self.__zbxcall(req)

    #
    # Problems.
    #
    def the_host_should_have_a_problem(self, host_name):
        host_id = self.__get_host_id(host_name)
        
        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "problem.get",
            "params": {
                "output": "extend",
                "hostids": [ "%s" ]
            }
        }
        """ % (self.__id, self.__auth, host_id)

        resp = self.__zbxcall(req)

        # Fail the test if the problem does not exist.
        if len(resp['result']) == 0:
            raise Exception("Host has no problems!")

    def the_host_should_have_no_problems(self, host_name):
        host_id = self.__get_host_id(host_name)
        
        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "problem.get",
            "params": {
                "output": "extend",
                "hostids": [ "%s" ]
            }
        }
        """ % (self.__id, self.__auth, host_id)

        resp = self.__zbxcall(req)

        # Fail the test if problems exist for the host.
        if len(resp['result']) != 0:
            raise Exception("Host has unresloved problems!")

    def __the_problem_should_have_a_tag_with_a_value(self, host_name, tag_name, tag_value):
        host_id = self.__get_host_id(host_name)
        
        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "problem.get",
            "params": {
                "output": "eventid",
                "selectTags": "extend",
                "hostids": [ "%s" ]
            }
        }
        """ % (self.__id, self.__auth, host_id)

        resp = self.__zbxcall(req)

        # Fail the test if the problem does not exist.
        if len(resp['result']) == 0:
            raise Exception("Host has no problems!")

        tags = resp['result'][0]['tags']
        for tv in tags:
            if tv['tag'] == tag_name and tv['value'] == tag_value:
                print()
                print("Tag witho no value found: %s" % (tag_name))
                return

        # Fail if the relevant tag was not found.
        raise Exception("Tag not found!")

    def the_problem_should_have_a_tag(self, host_name, tag_name):
        self.__the_problem_should_have_a_tag_with_a_value(host_name, tag_name, '')

    def the_problem_should_have_a_tag_with_a_value(self, host_name, tag_name, tag_value):
        self.__the_problem_should_have_a_tag_with_a_value(host_name, tag_name, tag_value)

    #
    # User macros.
    #
    def create_global_user_macro(self, macro_name, macro_value):
        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "usermacro.createglobal",
            "params":  {
                "macro": %s,
                "value": %s
            }
        }""" % (self.__id, self.__auth, json.dumps(macro_name), json.dumps(macro_value))
        
        print()
        print("Macro name: %s" % (macro_name))
        print("Macro value: %s" % (macro_value))
        print()

        resp = self.__zbxcall(req)
        
    def __get_global_user_macro_id(self, macro_name):      
        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "usermacro.get",
            "params": {
                "output": "globalmacroid",
                "globalmacro": true,
                "search": {
                    "macro": %s
                }  
            }
        }""" % (self.__id, self.__auth, json.dumps(macro_name))

        resp = self.__zbxcall(req)

        print()
        print("Macro name: %s" % (macro_name))
        
        # Macro exists?
        if len(resp['result']) == 0:
            raise Exception("Macro does not exist!")

        macro_id = resp['result'][0]['globalmacroid']
        
        print("Macro id: %s" % (macro_id))
        print()

        return macro_id

    def delete_global_user_macro(self, macro_name):
        macro_id = self.__get_global_user_macro_id(macro_name)
        
        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "usermacro.deleteglobal",
            "params": [
                "%s"
            ]
        }""" % (self.__id, self.__auth, macro_id)

        resp = self.__zbxcall(req)

    #
    # LLD rule
    #
    def create_lld_rule(self, host_name, rule_name, rule_key, rule_type):
        host_id       = self.__get_host_id(host_name)
        rule_type_num = self.__get_item_type_num(rule_type)

        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "discoveryrule.create",
            "params": {
                "hostid": "%s",
                "name": %s,
                "key_": %s,
                "type": "%d"

            }
        }""" % (self.__id, self.__auth, host_id, json.dumps(rule_name), json.dumps(rule_key),
                rule_type_num)

        resp = self.__zbxcall(req)

    def __get_lld_rule_id(self, host_name, rule_key):
        host_id = self.__get_host_id(host_name)
        
        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "discoveryrule.get",
            "params": {
                "output": "itemid",
                "hostids": "%s",
                "search": {
                    "key_": %s
                }            }
        }""" % (self.__id, self.__auth, host_id, json.dumps(rule_key))

        resp = self.__zbxcall(req)

        print()
        print("Host name: %s" % (host_name))
        print("Rule key: %s" % (rule_key))

        # Rule exists?
        if len(resp['result']) == 0:
            raise Exception("LLD rule does not exist!")

        rule_id = resp['result'][0]['itemid']
        
        print("Item id: %s" % (rule_id))
        print()

        return rule_id


    def delete_lld_rule(self, host_name, rule_key):
        rule_id = self.__get_lld_rule_id(host_name, rule_key)
        
        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "discoveryrule.delete",
            "params": [
                "%s"
            ]
        }""" % (self.__id, self.__auth, rule_id)

        resp = self.__zbxcall(req)

    #
    # LLD item prototypes.
    #
    def create_lld_item(self, host_name, rule_key, item_name, item_key, item_type, data_type):
        host_id       = self.__get_host_id(host_name)
        rule_id       = self.__get_lld_rule_id(host_name, rule_key)
        item_type_num = self.__get_item_type_num(item_type)
        data_type_num = self.__get_data_type_num(data_type)
        
        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "itemprototype.create",
            "params": {
                "hostid": "%s",
                "ruleid": "%s",
                "name": %s,
                "key_": %s,
                "type": %d,
                "value_type": %d
            }
        }""" % (self.__id, self.__auth, host_id, rule_id, json.dumps(item_name), json.dumps(item_key),
                item_type_num, data_type_num)

        resp = self.__zbxcall(req)

    def __get_lld_item_id(self, host_name, rule_key, item_key):
        host_id = self.__get_host_id(host_name)
        rule_id = self.__get_lld_rule_id(host_name, rule_key)

        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "itemprototype.get",
            "params": {
                "output": "itemid",
                "hostids": "%s",
                "discoveryids": "%s",
                "search": {
                    "key_": %s
                }            }
        }""" % (self.__id, self.__auth, host_id, rule_id, json.dumps(item_key))

        resp = self.__zbxcall(req)

        print()
        print("Host name: %s" % (host_name))
        print("LLD rule key: %s" % (rule_key))
        print("LLD item prototype key: %s" % (item_key))

        # Item exists?
        if len(resp['result']) == 0:
            raise Exception("Item does not exist!")

        item_id = resp['result'][0]['itemid']
        
        print("LLD item prototype id: %s" % (item_id))
        print()

        return item_id

    def delete_lld_item(self, host_name, rule_key, item_key):
        item_id = self.__get_lld_item_id(host_name, rule_key, item_key)
        
        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "itemprototype.delete",
            "params": [
                "%s"
            ]
        }""" % (self.__id, self.__auth, item_id)

        resp = self.__zbxcall(req)

    #
    # LLD trigger prototypes.
    #
    def create_lld_trigger(self, trigger_name, trigger_expression):
        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "triggerprototype.create",
            "params": {
                "description": %s,
                "expression": %s
            }
        }""" % (self.__id, self.__auth, json.dumps(trigger_name), json.dumps(trigger_expression))
        
        print()
        print("LLD trigger name: %s" % (trigger_name))
        print("LLD trigger expression: %s" % (trigger_expression))
        print()

        resp = self.__zbxcall(req)

    def __get_lld_trigger_id(self, trigger_name):      
        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "triggerprototype.get",
            "params": {
                "output": "triggerid",
                "search": {
                    "description": %s
                }
            }
        }""" % (self.__id, self.__auth, json.dumps(trigger_name))

        resp = self.__zbxcall(req)

        print()
        print("LLD trigger name: %s" % (trigger_name))
        
        # Trigger exists?
        if len(resp['result']) == 0:
            raise Exception("Trigger does not exist!")

        # Triggers are independent of hosts, items and thus can be "duplicated".
        if len(resp['result']) > 1:
            raise Exception("Multiple triggers with same name!!")

        trigger_id = resp['result'][0]['triggerid']
        
        print("LLD trigger id: %s" % (trigger_id))
        print()

        return trigger_id

    def set_lld_trigger_expression(self, trigger_name, trigger_expression):
        trigger_id = self.__get_lld_trigger_id(trigger_name);
        
        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "triggerprototype.update",
            "params": {
                "triggerid": "%s",
                "expression": %s
            }
        }""" % (self.__id, self.__auth, trigger_id, json.dumps(trigger_expression))

        resp = self.__zbxcall(req)

    def set_lld_trigger_correlation_mode_and_tag(self, trigger_name, tag_name):
        trigger_id = self.__get_lld_trigger_id(trigger_name);
        
        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "triggerprototype.update",
            "params": {
                "triggerid": "%s",
                "type": "1",
                "correlation_mode": "1",
                "correlation_tag": %s
            }
        }""" % (self.__id, self.__auth, trigger_id, json.dumps(tag_name))

        resp = self.__zbxcall(req)

    def __get_lld_trigger_tags(self, trigger_name):
        trigger_id = self.__get_lld_trigger_id(trigger_name)

        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "triggerprototype.get",
            "params": {
                "output": "triggerid",
                "selectTags": "extend",
                "triggerids": [ "%s" ]
            }
        }""" % (self.__id, self.__auth, trigger_id)

        resp = self.__zbxcall(req)

        return resp['result'][0]['tags']

    def __add_tag_with_value_to_lld_trigger(self, trigger_name, tag_name, tag_value):
        trigger_id = self.__get_lld_trigger_id(trigger_name)

        tags = self.__get_lld_trigger_tags(trigger_name)

        # Add the new tag to the list of existing tags.
        # Otherwise the list of tags for the trigger is reset and contains
        # only the new tag.
        tv = {'tag': tag_name, 'value': tag_value}
        tags.append(tv)
        
        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "triggerprototype.update",
            "params": {
                "triggerid": "%s",
                "tags": %s
            }
        }""" % (self.__id, self.__auth, trigger_id, json.dumps(tags))

        resp = self.__zbxcall(req)

    def add_tag_to_lld_trigger(self, trigger_name, tag_name):
        self.__add_tag_with_value_to_lld_trigger(trigger_name, tag_name, "")

    def add_tag_with_value_to_lld_trigger(self, trigger_name, tag_name, tag_value):
        self.__add_tag_with_value_to_lld_trigger(trigger_name, tag_name, tag_value)

    def delete_lld_trigger(self, trigger_name):
        trigger_id = self.__get_lld_trigger_id(trigger_name)
        
        req = """
        {
            "jsonrpc": "2.0",
            "id": %d,
            "auth": "%s",
            "method": "triggerprototype.delete",
            "params": [
                "%s"
            ]
        }""" % (self.__id, self.__auth, trigger_id)

        resp = self.__zbxcall(req)
