#
# A hack-and-slash attempt at testing ZBXNEXT-3457 automatically.
#
# Design guidelines of the tests:
# * should read as test description;
# * low-level implementation details hidden in test library;
# * common configuratin abstracted in setup and teardown;
#   * configured hosts, triggers, items, etc. have "default" values that can be changed
#     by the test.
# * tests are independent of each other;
#
# Design guidelines for the library and keywords:
# * be at the same level as UI user, e.g. no IDs!!!;
# * performance and efficiency is not important, clarity is;
# * code repetition is encouraged to decouple keywords from each other;
#   * if abstraction breaks the JSON into multiple parts then it is not allowed;
# * log enough information to assist in troubleshooting.
#
# Instructions:
# * use Python 3 and Robot 3;
# * put the ZbxLib.py in lib/ subdirectory.
#
# Let me know in case of questions: sandis.neilands@zabbix.com.
#
*** Settings ***
Library  lib/ZbxLib.py  http://192.168.3.252/zbxnext_3457/api_jsonrpc.php

Suite Setup     Start User Session
Suite Teardown  Stop User Session

*** Variables ***
${ZABBIX_SERVER}             192.168.3.252
${ZABBIX_SENDER}             /home/sandis/ZBXNEXT/ZBXNEXT-3457/ZBXNEXT-3457/usr/bin/zabbix_sender

# CacheUpdateFrequency in zabbix_server.conf set to 5.
${SERVER_CACHE_UPDATE_FREQ}  7

${API_VERSION}      3.3.0

${USER}             Admin
${PASSWORD}         zabbix

${HOST_GROUP_NAME}  zbxnext_3457_servers
${HOST_NAME}        zbxnext_3457_host

${ITEM_NAME}        zbxnext_3457_item
${ITEM_KEY}         ${ITEM_NAME}
${ITEM_TYPE}        Zabbix trapper
${ITEM_DATA_TYPE}   text

${TRIGGER_NAME}                 zbxnext_3457_trigger
# Dummy expression, must be replaced by test.
${TRIGGER_EXPRESSION}           {${HOST_NAME}:${ITEM_KEY}.now()}<>0

${LLD_RULE_NAME}        zbxnext_3457_lld_rule
${LLD_RULE_KEY}         ${LLD_RULE_NAME}
${LLD_RULE_TYPE}        Zabbix trapper

${LLD_ITEM_NAME}        key[{#LLDKEY}]
${LLD_ITEM_KEY}         ${LLD_ITEM_NAME}
${LLD_ITEM_TYPE}        Zabbix trapper
${LLD_ITEM_DATA_TYPE}   text

${LLD_TRIGGER_NAME}                 {#LLDKEY}_trigger
# Dummy expression, must be replaced by test.
${LLD_TRIGGER_EXPRESSION}           {${HOST_NAME}:key[{#LLDKEY}].now()}<>0

*** Test Cases ***
Empty LLD Macro In Tag, Value And Correlation Tag Are Not Allowed
    [Documentation]  See ZBX-11349.
    [Setup]  Create Host With LLD Item And Trigger

    # Configure LLD trigger.
    Set LLD Trigger Expression                 ${LLD TRIGGER_NAME}  {${HOST_NAME}:${LLD_ITEM_KEY}.regexp(".*zbx.*",#1)}=1
    Set LLD Trigger Correlation Mode And Tag   ${LLD_TRIGGER_NAME}  {#LLDEMPTY}
    Add Tag With Value To LLD Trigger          ${LLD_TRIGGER_NAME}  {#LLDEMPTY}  {#LLDEMPTY}

    Wait for Configuration Sync                ${SERVER_CACHE_UPDATE_FREQ}

    # Send in LLD data so that item and trigger is created from prototypes (should fail - empty tag and correlation tag not allowed).
    Sending Item Value To Server Should Fail   ${HOST_NAME}  ${LLD_RULE_KEY}  { "data": [ { "{#LLDKEY}": "/", "{#LLDEMPTY}": "" } ] }

    [Teardown]  Delete Created Host

LLD Macro Expanding To Slash In User Macro Context
    [Documentation]  Case 1: In tag name and tag value it must be possible to use {$MACRO:context} where
    ...                      context is LLD macro and forward slash is allowed in macro

    [Setup]  Create Host With LLD Item And Trigger

    # Configure user macro.
    Create Global User Macro               {$ROOT_PARTITION:/}  root_partition

    # Configure LLD trigger.
    Set LLD Trigger Expression                 ${LLD TRIGGER_NAME}  {${HOST_NAME}:${LLD_ITEM_KEY}.regexp(".*zbx.*",#1)}=1
    Set LLD Trigger Correlation Mode And Tag   ${LLD_TRIGGER_NAME}  {$ROOT_PARTITION:"{#LLDKEY}"}
    Add Tag With Value To LLD Trigger          ${LLD_TRIGGER_NAME}  {$ROOT_PARTITION:"{#LLDKEY}"}  {$ROOT_PARTITION:"{#LLDKEY}"}

    Wait for Configuration Sync                ${SERVER_CACHE_UPDATE_FREQ}

    # Send in LLD data so that item and trigger is created from prototypes.
    Send Item Value To Server                  ${HOST_NAME}  ${LLD_RULE_KEY}  { "data": [ { "{#LLDKEY}": "/" } ] }

    # Wait until server gets the discovered item, trigger.
    Wait for Configuration Sync                ${SERVER_CACHE_UPDATE_FREQ}

    # Cause a problem and check macro expansion. key[/] discovered from key[{#LLDKEY}].
    Send Item Value To Server                   ${HOST_NAME}  key[/]  zbx
    The Host Should Have A Problem              ${HOST_NAME}
    The Problem Should Have A Tag With A Value  ${HOST_NAME}  root_partition  root_partition

    # The problem is recovered since tag matches.
    Send Item Value To Server                   ${HOST_NAME}  key[/]  abc
    The Host Should Have No Problems            ${HOST_NAME}

    [Teardown]  Run Keywords  Delete Created Host
    ...         AND           Delete Global User Macro  {$ROOT_PARTITION:/}

LLD Macro Expanding To Forward Slash Is Allowed In Tag Name And Value
    [Documentation]  Case 2: Tag name with forward slash is allowed and error in frontend is not shown
    ...              Case 2a: Same rules apply to "Tag for matching" as for Tag name and you do not get error in frontend

    [Setup]  Create Host With LLD Item And Trigger

    # Configure LLD trigger.
    Set LLD Trigger Expression                 ${LLD TRIGGER_NAME}  {${HOST_NAME}:${LLD_ITEM_KEY}.regexp(".*zbx.*",#1)}=1
    Set LLD Trigger Correlation Mode And Tag   ${LLD_TRIGGER_NAME}  {#LLDKEY}
    Add Tag With Value To LLD Trigger          ${LLD_TRIGGER_NAME}  {#LLDKEY}  {#LLDKEY}

    Wait for Configuration Sync                ${SERVER_CACHE_UPDATE_FREQ}

    # Send in LLD data so that item and trigger is created from prototypes.
    Send Item Value To Server                   ${HOST_NAME}  ${LLD_RULE_KEY}  { "data": [ { "{#LLDKEY}": "/" } ] }

    # Wait until server gets the discovered item, trigger.
    Wait for Configuration Sync                 ${SERVER_CACHE_UPDATE_FREQ}

    # Cause a problem and check macro expansion. key[/] discovered from key[{#LLDKEY}].
    Send Item Value To Server                   ${HOST_NAME}  key[/]  zbx
    The Host Should Have A Problem              ${HOST_NAME}
    The Problem Should Have A Tag With A Value  ${HOST_NAME}  /  /

    # The problem is recovered since tag matches.
    Send Item Value To Server                   ${HOST_NAME}  key[/]  abc
    The Host Should Have No Problems            ${HOST_NAME}

    [Teardown]  Delete Created Host

User Macro With Forward Slash Context Is Allowed
    [Documentation]  Case 3: User macro with forward slash context is allowed

    [Setup]  Create Host With Item And Trigger

    # Configure macro.
    Create Global User Macro               {$ROOT_PARTITION:"/"}  root_partition

    # Configure trigger.
    Set Trigger Expression                 ${TRIGGER_NAME}  {${HOST_NAME}:${ITEM_NAME}.regexp(".*zbx.*",#1)}=1
    Set Trigger Correlation Mode And Tag   ${TRIGGER_NAME}  {$ROOT_PARTITION:"/"}
    Add Tag With Value To Trigger          ${TRIGGER_NAME}  {$ROOT_PARTITION:"/"}  {$ROOT_PARTITION:"/"}

    Wait for Configuration Sync            ${SERVER_CACHE_UPDATE_FREQ}

    # Cause a problem and check macro expansion.
    Send Item Value To Server                   ${HOST_NAME}  ${ITEM_KEY}  zbx
    The Host Should Have A Problem              ${HOST_NAME}

    The Problem Should Have A Tag With A Value  ${HOST_NAME}  root_partition  root_partition

    # The problem is recovered since tag matches.
    Send Item Value To Server                   ${HOST_NAME}  ${ITEM_KEY}  abc
    The Host Should Have No Problems            ${HOST_NAME}

    [Teardown]  Run Keywords  Delete Created Host
    ...         AND           Delete Global User Macro  {$ROOT_PARTITION:"/"}

Tag Value With Forward Slash Is Allowed
    [Documentation]  Case 4a: Tag value with forward slash is allowed

    [Setup]  Create Host With Item And Trigger

    # Configure trigger.
    Set Trigger Expression                 ${TRIGGER_NAME}  {${HOST_NAME}:${ITEM_NAME}.regexp(".*zbx.*",#1)}=1
    Set Trigger Correlation Mode And Tag   ${TRIGGER_NAME}  root
    Add Tag With Value To Trigger          ${TRIGGER_NAME}  root  /root

    Wait for Configuration Sync            ${SERVER_CACHE_UPDATE_FREQ}

    # Cause a problem and check macro expansion.
    Send Item Value To Server                   ${HOST_NAME}  ${ITEM_KEY}  zbx
    The Host Should Have A Problem              ${HOST_NAME}

    The Problem Should Have A Tag With A Value  ${HOST_NAME}  root  /root

    # The problem is recovered since tag matches.
    Send Item Value To Server                   ${HOST_NAME}  ${ITEM_KEY}  abc
    The Host Should Have No Problems            ${HOST_NAME}

    [Teardown]  Delete Created Host


Tag Name And Value With Forward Slash In Regsub Is Allowed
    [Documentation]  Case 4b: Tag name and value with forward slash in regsub is allowed

    [Setup]  Create Host With Item And Trigger

    # Configure trigger.
    Set Trigger Expression                 ${TRIGGER_NAME}  {${HOST_NAME}:${ITEM_NAME}.regexp(".*zbx.*",#1)}=1
    Set Trigger Correlation Mode And Tag   ${TRIGGER_NAME}  {{ITEM.VALUE}.regsub("(/.*)",\\1)}
    Add Tag With Value To Trigger          ${TRIGGER_NAME}  {{ITEM.VALUE}.regsub("(/.*)",\\1)}  {{ITEM.VALUE}.regsub("(/.*)",\\1)}

    Wait for Configuration Sync            ${SERVER_CACHE_UPDATE_FREQ}

    # Cause a problem and check macro expansion.
    Send Item Value To Server                   ${HOST_NAME}  ${ITEM_KEY}  /zbx
    The Host Should Have A Problem              ${HOST_NAME}
    The Problem Should Have A Tag With A Value  ${HOST_NAME}  /zbx  /zbx

    # The problem is not recovered since tag doesn't match
    Send Item Value To Server                   ${HOST_NAME}  ${ITEM_KEY}  abc
    The Host Should Have A Problem              ${HOST_NAME}
    The Problem Should Have A Tag With A Value  ${HOST_NAME}  /zbx  /zbx

    [Teardown]  Delete Created Host

Slash Can Be Used In Tag Name And Tag Value
    [Documentation]  Case 4c: Tag name and value with forward slash in user macros is allowed and tags are added

    [Setup]  Create Host With Item And Trigger

    # Configure macro.
    Create Global User Macro               {$SLASH_MACRO}  /

    # Configure trigger.
    Set Trigger Expression                 ${TRIGGER_NAME}  {${HOST_NAME}:${ITEM_NAME}.regexp(".*zbx.*",#1)}=1
    Set Trigger Correlation Mode And Tag   ${TRIGGER_NAME}  {$SLASH_MACRO}
    Add Tag With Value To Trigger          ${TRIGGER_NAME}  {$SLASH_MACRO}  {$SLASH_MACRO}

    Wait for Configuration Sync            ${SERVER_CACHE_UPDATE_FREQ}

    # Cause a problem and check macro expansion.
    Send Item Value To Server                   ${HOST_NAME}  ${ITEM_KEY}  zbx
    The Host Should Have A Problem              ${HOST_NAME}

    The Problem Should Have A Tag With A Value  ${HOST_NAME}  /  /

    # The problem is recovered since tag matches.
    Send Item Value To Server                   ${HOST_NAME}  ${ITEM_KEY}  abc
    The Host Should Have No Problems            ${HOST_NAME}

    [Teardown]  Run Keywords  Delete Created Host
    ...         AND           Delete Global User Macro  {$SLASH_MACRO}

Slash Can Be Used As Correlation Tag
    [Documentation]  Case 4d: Tag for matching with forward slash in user macros is allowed and tags are added

    [Setup]  Create Host With Item And Trigger

    # Configure macro.
    Create Global User Macro               {$SLASH_MACRO}  /

    # Configure trigger.
    Set Trigger Expression                 ${TRIGGER_NAME}  {${HOST_NAME}:${ITEM_NAME}.regexp(".*zbx.*",#1)}=1
    Set Trigger Correlation Mode And Tag   ${TRIGGER_NAME}  {$SLASH_MACRO}
    Add Tag With Value To Trigger          ${TRIGGER_NAME}  test            test

    Wait for Configuration Sync            ${SERVER_CACHE_UPDATE_FREQ}

    # Cause a problem and check macro expansion.
    Send Item Value To Server                   ${HOST_NAME}  ${ITEM_KEY}  zbx
    The Host Should Have A Problem              ${HOST_NAME}
    The Problem Should Have A Tag With A Value  ${HOST_NAME}  test  test

    # The problem is not recovered since tag doesn't match
    Send Item Value To Server                   ${HOST_NAME}  ${ITEM_KEY}  abc
    The Host Should Have A Problem              ${HOST_NAME}
    The Problem Should Have A Tag With A Value  ${HOST_NAME}  test  test

    [Teardown]  Run Keywords  Delete Created Host
    ...         AND           Delete Global User Macro  {$SLASH_MACRO}

User Macro Can Be Used In Tag Name And Tag Value
    [Documentation]  Case 5: In tag name and tag value it must be possible to use {$MACRO}

    [Setup]  Create Host With Item And Trigger

    # Configure.
    Set Trigger Expression                 ${TRIGGER_NAME}            {${HOST_NAME}:${ITEM_NAME}.regexp(".*zbx.*",#1)}=1
    Create Global User Macro               {$ZABBIX_SERVER_NAME}      zabbix
    Create Global User Macro               {$ZABBIX_SERVER_PRIORITY}  high
    Add Tag With Value To Trigger          ${TRIGGER_NAME}            {$ZABBIX_SERVER_NAME}  {$ZABBIX_SERVER_PRIORITY}

    Wait for Configuration Sync            ${SERVER_CACHE_UPDATE_FREQ}

    # Cause a problem and check macro expansion.
    Send Item Value To Server                   ${HOST_NAME}  ${ITEM_KEY}  zbx
    The Host Should Have A Problem              ${HOST_NAME}
    The Problem Should Have A Tag With A Value  ${HOST_NAME}  zabbix  high

    # Recover the problem.
    Send Item Value To Server                   ${HOST_NAME}  ${ITEM_KEY}  abc
    The Host Should Have No Problems            ${HOST_NAME}

    [Teardown]  Run Keywords  Delete Created Host
    ...         AND           Delete Global User Macro  {$ZABBIX_SERVER_NAME}
    ...         AND           Delete Global User Macro  {$ZABBIX_SERVER_PRIORITY}


A Problem Is Recovered By Expression
    [Setup]  Create Host With Item And Trigger

    Set Trigger Expression                 ${TRIGGER_NAME}  {${HOST_NAME}:${ITEM_NAME}.regexp(".*zbx.*",#1)}=1
    Wait for Configuration Sync            ${SERVER_CACHE_UPDATE_FREQ}

    Send Item Value To Server              ${HOST_NAME}  ${ITEM_KEY}  zbx
    The Host Should Have A Problem         ${HOST_NAME}

    Send Item Value To Server              ${HOST_NAME}  ${ITEM_KEY}  abc
    The Host Should Have No Problems       ${HOST_NAME}

    [Teardown]  Delete Created Host

A Problem With Various Tags Is Recovered By Expression
    [Setup]  Create Host With Item And Trigger

    #
    # Configure the trigger.
    #
    Set Trigger Expression                 ${TRIGGER_NAME}  {${HOST_NAME}:${ITEM_NAME}.regexp(".*zbx.*",#1)}=1

    Add Tag To Trigger                     ${TRIGGER_NAME}  Tag
    Add Tag To Trigger                     ${TRIGGER_NAME}  /
    Add Tag To Trigger                     ${TRIGGER_NAME}  {$UNRESOLVED_MACRO}
    Add Tag To Trigger                     ${TRIGGER_NAME}  {$UNRESOLVED_CMACRO:"/"}

    Add Tag With Value To Trigger          ${TRIGGER_NAME}  TagName                   TagValue
    Add Tag With Value To Trigger          ${TRIGGER_NAME}  Tag/Name                  Tag/Value
    Add Tag With Value To Trigger          ${TRIGGER_NAME}  {$UNRESOLVED_NAME}        {$UNRESOLVED_VALUE}
    Add Tag With Value To Trigger          ${TRIGGER_NAME}  {$UNRESOLVED_CNAME:"/"}   {$UNRESOLVED_CVALUE:"/"}

    Wait for Configuration Sync            ${SERVER_CACHE_UPDATE_FREQ}

    #
    # Test body.
    #
    Send Item Value To Server              ${HOST_NAME}  ${ITEM_KEY}  zbx
    The Host Should Have A Problem         ${HOST_NAME}

    The Problem Should Have A Tag          ${HOST_NAME}  Tag
    The Problem Should Have A Tag          ${HOST_NAME}  /
    The Problem Should Have A Tag          ${HOST_NAME}  {$UNRESOLVED_MACRO}
    The Problem Should HAve A Tag          ${HOST_NAME}  {$UNRESOLVED_CMACRO:"/"}

    The Problem Should Have A Tag With A Value  ${HOST_NAME}  TagName                   TagValue
    The Problem Should Have A Tag With A Value  ${HOST_NAME}  Tag/Name                  Tag/Value
    The Problem Should Have A Tag With A Value  ${HOST_NAME}  {$UNRESOLVED_NAME}        {$UNRESOLVED_VALUE}
    The Problem Should Have A Tag With A Value  ${HOST_NAME}  {$UNRESOLVED_CNAME:"/"}   {$UNRESOLVED_CVALUE:"/"}

    Send Item Value To Server              ${HOST_NAME}  ${ITEM_KEY}  abc
    The Host Should Have No Problems       ${HOST_NAME}

    [Teardown]  Delete Created Host

*** Keywords ***
Send Item Value To Server
    [Arguments]  ${host_name}  ${item_key}  ${item_value}

    Call Zabbix Sender And Expect Success  ${ZABBIX_SENDER}  ${ZABBIX_SERVER}  ${host_name}  ${item_key}  ${item_value}

Sending Item Value To Server Should Fail
    [Arguments]  ${host_name}  ${item_key}  ${item_value}

    Call Zabbix Sender And Expect Failure  ${ZABBIX_SENDER}  ${ZABBIX_SERVER}  ${host_name}  ${item_key}  ${item_value}

Create Host With Item And Trigger
    # Modify the items, triggers in tests.
    Create Host Group  ${HOST_GROUP_NAME}
    Create Host        ${HOST_NAME}        ${HOST_GROUP_NAME}
    Create Item        ${HOST_NAME}        ${ITEM_NAME}           ${ITEM_KEY}  ${ITEM_TYPE}  ${ITEM_DATA_TYPE}
    Create Trigger     ${TRIGGER_NAME}     ${TRIGGER_EXPRESSION}

Create Host With LLD Item And Trigger
    # Modify the items, triggers in tests.
    Create Host Group      ${HOST_GROUP_NAME}
    Create Host            ${HOST_NAME}         ${HOST_GROUP_NAME}

    Create LLD Rule        ${HOST_NAME}         ${LLD_RULE_NAME}    ${LLD_RULE_KEY}  ${LLD_RULE_TYPE}

    Create LLD Item        ${HOST_NAME}         ${LLD_RULE_KEY}  ${LLD_ITEM_NAME}  ${LLD_ITEM_KEY}  ${LLD_ITEM_TYPE}  ${LLD_ITEM_DATA_TYPE}
    Create LLD Trigger     ${LLD_TRIGGER_NAME}  ${LLD_TRIGGER_EXPRESSION}

Delete Created Host
    Delete Host        ${HOST_NAME}
    Delete Host Group  ${HOST_GROUP_NAME}

Start User Session
    API Version Is     ${API_VERSION}
    Login              ${USER}             ${PASSWORD}

Stop User Session
    Logout
